#!/usr/bin/python

#python script by Tak Lee for 2016 BIML network analysis 
#Usage: to find context-assoicated hubs
#$python CAH_find.py [input DEGs] [input network] [input network hublist] [input annotation terms] > [output file]

import os,sys
from scipy import stats
from operator import itemgetter


cnt_neighbor = {}
p_values = {}

#read in DEGs
degs = [deg for deg in ((l.strip()) for l in open(sys.argv[1],'r'))]
deg_num = len(degs)

#open network to find direct neighborhoods
network = [sorted((g1,g2)) for (g1,g2,s) in ((l2.strip()).split('\t') for l2 in open(sys.argv[2],'r'))]

#read in hubgenes
hubs = {hub:links for (hub,links) in ((l3.strip()).split('\t') for l3 in open(sys.argv[3],'r'))}

#read in GOterms
goterms = {termgenes:terms for (termgenes,terms) in ((l4.strip()).split('\t') for l4 in open(sys.argv[4],'r'))}

#cnt the number of neighboring genes that is also a DEG
for (g1,g2) in network:

	if g1 in hubs:
		if g1 not in cnt_neighbor:
			cnt_neighbor[g1] = 0
		else:
			if g2 in degs:
				cnt_neighbor[g1] += 1
	if g2 in hubs:
		if g2 not in cnt_neighbor:
			cnt_neighbor[g2] = 0
		else:
			if g1 in degs:
				cnt_neighbor[g2] += 1
#calculate p-value
for hub_genes in cnt_neighbor:
	a = int(cnt_neighbor[hub_genes]) #number of genes that are DEGs and also neighbor of hubs
	b = int(hubs[hub_genes]) - a     #number of genes that are hub neighbors but not DEGs
	c = int(deg_num) - a		   #number of genes that are DEGs but not hub neighbors
	#total number of mouse genes = 20058
	d = 36736 - a - b - c	   #number of genes that are not DEGs and also not hub neighbors		
	if(a>0):
		oddsratio, pvalue = stats.fisher_exact([[a,b],[c,d]]) 
		if(pvalue <= 0.01):
			p_values[hub_genes] = pvalue
#print output
for candidate_gene,p in sorted(p_values.items(),key=itemgetter(1),reverse=False):
	if candidate_gene in goterms:
		print "%s\t%s\t%s" %(str(candidate_gene),goterms[candidate_gene],str(p))
	else:
		print "%s\t%s\t%s" %(str(candidate_gene),'NA',str(p))
